 /*
 * Copyright (C) 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
*/

// ContactSettings.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "ContactSettings.h"
#include "MainSyncFrm.h"
#include "ClientUtil.h"

#include "winmaincpp.h"
#include "utils.h"
#include "comutil.h"
#include "OutlookPlugin.h"
#include <string>

using namespace std;
// CContactSettings

static wstring outlookFolder;
static CContactSettings* wndContacts;
static HANDLE handleThread;

IMPLEMENT_DYNCREATE(CContactSettings, CDialog)

CContactSettings::CContactSettings()
	: CDialog(CContactSettings::IDD)
{
    handleThread = NULL;
}

CContactSettings::~CContactSettings()
{
    // clean stuff used in the select Outlook folder thread
    wndContacts = NULL;
    if (handleThread) {
        TerminateThread(handleThread, -1);
        CloseHandle(handleThread);
        handleThread = NULL;
    }
}

void CContactSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_CONTACTS_COMBO_SYNCTYPE, lstSyncType);
    DDX_Control(pDX, IDC_CONTACTS_EDIT_FOLDER, editFolder);
    DDX_Control(pDX, IDC_CONTACTS_CHECK_INCLUDE, checkInclude);
    DDX_Control(pDX, IDC_CONTACTS_BUT_FOLDER, butSelectFolder);
    DDX_Control(pDX, IDC_CONTACTS_EDIT_REMOTE, editRemote);
    DDX_Control(pDX, IDC_CONTACTS_RADIO_SIF, radioSif);
    DDX_Control(pDX, IDC_CONTACTS_RADIO_VCARD, radioVcard);
    DDX_Control(pDX, IDC_CONTACTS_GROUP_DIRECTION, groupDirection);
    DDX_Control(pDX, IDC_CONTACTS_GROUP_FOLDER, groupFolder);
    DDX_Control(pDX, IDC_CONTACTS_GROUP_ADVANCED, groupAdvanced);
}

BEGIN_MESSAGE_MAP(CContactSettings, CDialog)
    ON_BN_CLICKED(IDC_CONTACTS_BUTOK, &CContactSettings::OnBnClickedContactsButok)
    ON_BN_CLICKED(IDC_CONTACTS_BUTCANCEL, &CContactSettings::OnBnClickedContactsButcancel)
    ON_BN_CLICKED(IDC_CONTACTS_BUT_FOLDER, &CContactSettings::OnBnClickedContactsButFolder)
END_MESSAGE_MAP()


// CContactSettings diagnostics

#ifdef _DEBUG
void CContactSettings::AssertValid() const
{
	CDialog::AssertValid();
}

#ifndef _WIN32_WCE
void CContactSettings::Dump(CDumpContext& dc) const
{
	CDialog::Dump(dc);
}
#endif
#endif //_DEBUG


// CContactSettings message handlers

BOOL CContactSettings::OnInitDialog(){
    CString s1;
    s1.LoadString(IDS_CONTACTS_DETAILS); SetWindowText(s1);
    CDialog::OnInitDialog();

    WindowsSyncSourceConfig* ssconf = ((OutlookConfig*)getConfig())->getSyncSourceConfig(CONTACT_);

    editFolder.SetLimitText(EDIT_TEXT_MAXLENGTH);
    editRemote.SetLimitText(EDIT_TEXT_MAXLENGTH);

    // load string resources
    s1.LoadString(IDS_SYNCTYPE1); lstSyncType.AddString(s1);
    s1.LoadString(IDS_SYNCTYPE2); lstSyncType.AddString(s1);
    s1.LoadString(IDS_SYNCTYPE3); lstSyncType.AddString(s1);

    s1.LoadString(IDS_SYNC_DIRECTION); SetDlgItemText(IDC_CONTACTS_GROUP_DIRECTION, s1);
    s1.LoadString(IDS_CONTACTS_FOLDER); SetDlgItemText(IDC_CONTACTS_GROUP_FOLDER, s1);
    s1.LoadString(IDS_CURRENT); SetDlgItemText(IDC_CONTACTS_STATIC_FOLDER, s1);
    s1.LoadString(IDS_INCLUDE_SUBFOLDERS); SetDlgItemText(IDC_CONTACTS_CHECK_INCLUDE, s1);
    s1.LoadString(IDS_SELECT_FOLDER); SetDlgItemText(IDC_CONTACTS_BUT_FOLDER, s1);
    s1.LoadString(IDS_REMOTE_NAME); SetDlgItemText(IDC_CONTACTS_STATIC_REMOTE, s1);

    s1.LoadString(IDS_ADVANCED); SetDlgItemText(IDC_CONTACTS_GROUP_ADVANCED, s1);
    s1.LoadString(IDS_DATA_FORMAT); SetDlgItemText(IDC_CONTACTS_STATIC_DATAFORMAT, s1);
    s1.LoadString(IDS_USE_SIF); SetDlgItemText(IDC_CONTACTS_RADIO_SIF, s1);
    s1.LoadString(IDS_USE_VCARD); SetDlgItemText(IDC_CONTACTS_RADIO_VCARD, s1);

    s1.LoadString(IDS_OK); SetDlgItemText(IDC_CONTACTS_BUTOK, s1);
    s1.LoadString(IDS_CANCEL); SetDlgItemText(IDC_CONTACTS_BUTCANCEL, s1);

    // load settings from Registry

    lstSyncType.SetCurSel(getSyncTypeIndex(ssconf->getSync()));
    
    // Get folder path.
    // Note: use 'toWideChar' because we need UTF-8 conversion.
    WCHAR* olFolder = toWideChar(ssconf->getFolderPath());
    s1 = olFolder;
    delete [] olFolder;
    try {
        if(s1 == ""){
            s1 = getDefaultFolderPath(CONTACT).data();
        }
    }
    catch (...){
        // an exception occured while trying to get the default folder
        EndDialog(-1);
    }

    SetDlgItemText(IDC_CONTACTS_EDIT_FOLDER, s1);
    
    if(ssconf->getUseSubfolders())
        checkInclude.SetCheck(BST_CHECKED);

    // Note: use 'toWideChar' because we need UTF-8 conversion.
    WCHAR* remName = toWideChar(ssconf->getURI());
    s1 = remName;
    delete [] remName;
    SetDlgItemText(IDC_CONTACTS_EDIT_REMOTE, s1);

    if( strstr(ssconf->getType(),"sif") ){
        radioSif.SetCheck(BST_CHECKED);
    }
    else
        radioVcard.SetCheck(BST_CHECKED);

    // disable windows xp theme, otherwise any color setting for groupbox
    // will be overriden by the theme settings
    if(((COutlookPluginApp*)AfxGetApp())->hLib){
        PFNSETWINDOWTHEME pfnSetWindowTheme =
            (PFNSETWINDOWTHEME)GetProcAddress(((COutlookPluginApp*)AfxGetApp())->hLib, "SetWindowTheme");
        pfnSetWindowTheme (groupDirection.m_hWnd,L" ",L" ");
        pfnSetWindowTheme (groupFolder.m_hWnd,L" ",L" ");
        pfnSetWindowTheme (groupAdvanced.m_hWnd,L" ",L" ");
    };

    // Accessing Outlook, could be no more in foreground
    SetForegroundWindow();

    wndContacts = this;
    return FALSE;
}
void CContactSettings::OnBnClickedContactsButok()
{
    // OK Button
    if(saveSettings(false)){
        CDialog::OnOK();
    };
}

void CContactSettings::OnBnClickedContactsButcancel()
{
    // Never read from winreg, will save when 'OK' is clicked on SyncSettings.
    //getConfig()->read();
    CDialog::OnCancel();
}

bool CContactSettings::saveSettings(bool saveToDisk)
{
    CString remoteName, outlookFolder, syncType;
    CString s1;
    _bstr_t bst;
    bool useSif;
    WindowsSyncSourceConfig* ssconf = ((OutlookConfig*)getConfig())->getSyncSourceConfig(CONTACT_);

    GetDlgItemText(IDC_CONTACTS_EDIT_REMOTE, remoteName);
    GetDlgItemText(IDC_CONTACTS_EDIT_FOLDER, outlookFolder);

    // change values
    if(remoteName == ""){
        s1.LoadString(IDS_ERROR_SET_REMOTE_NAME);
        MessageBox(s1);
        return false;
    };
    // sync source enabled
    ssconf->setSync(getSyncTypeName(lstSyncType.GetCurSel()));

    // Note: use 'toMultibyte' which uses charset UTF-8.
    //       (when writing to winreg, toWideChar is then called)
    char* olFolder = toMultibyte(outlookFolder.GetBuffer());
    if (olFolder) {
        ssconf->setFolderPath(olFolder);
        delete [] olFolder;
    }

    if(checkInclude.GetCheck() == BST_CHECKED)
        ssconf->setUseSubfolders(true);
    else
        ssconf->setUseSubfolders(false);

    char* remName = toMultibyte(remoteName.GetBuffer());
    if (remName) {
        ssconf->setURI(remName);
        delete [] remName;
    }

    if(radioSif.GetCheck() == BST_CHECKED)
        useSif = true;
    else
        useSif = false;

    // TODO: data formats
    if(useSif){
        ssconf->setType("text/x-s4j-sifc");
        ssconf->setVersion("");                     // TODO: Change when SIF version is ready - get from define.
        ssconf->setEncoding("b64");
    }
    else{
        ssconf->setType("text/x-vcard"); 
        ssconf->setVersion(VCARD_VERSION_SUPPORTED);
        // When encryption is used, encoding is always 'base64'.
        if ( !strcmp(ssconf->getEncryption(), "") ) {
        ssconf->setEncoding("bin");
        }
        else {
            ssconf->setEncoding("b64");
        }
    }

    // Never save to winreg, will save when 'OK' is clicked on SyncSettings.
    //if(saveToDisk)
    //    ((OutlookConfig*)getConfig())->save();

    return true;
}

int pickFolderContacts(){

    CString s1;
    try {
        outlookFolder = pickOutlookFolder(CONTACT);
    }
    catch (...) {
        printLog("Exception thrown by pickOutlookFolder", LOG_DEBUG);
        outlookFolder = L"";
    }

    if (wndContacts) {      // dialog could have been closed...
        if (outlookFolder != L""){
            s1 = outlookFolder.data();
            wndContacts->SetDlgItemText(IDC_CONTACTS_EDIT_FOLDER, s1);
        }
        wndContacts->SetForegroundWindow();
        wndContacts->EndModalState();
    }
    return 0;
}

void CContactSettings::OnBnClickedContactsButFolder(){
    outlookFolder = L"";
    BeginModalState();
    handleThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)pickFolderContacts, NULL, 0, NULL);
}
