 /*
 * Copyright (C) 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
*/

// ProxySettings.cpp : implementation file
//

#include "stdafx.h"
#include "OutlookPlugin.h"
#include "ProxySettings.h"
#include "ClientUtil.h"
#include "winmaincpp.h"


// CProxySettings dialog

IMPLEMENT_DYNAMIC(CProxySettings, CDialog)

CProxySettings::CProxySettings(CWnd* pParent /*=NULL*/)
	: CDialog(CProxySettings::IDD, pParent)
{

}

CProxySettings::~CProxySettings()
{
}

void CProxySettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_PROXY_ENABLE, checkEnable);
    DDX_Control(pDX, IDC_PROXY_AUTH, checkAuth);
    DDX_Control(pDX, IDC_PROXY_EDIT_USER, editUser);
    DDX_Control(pDX, IDC_PROXY_EDIT_PASS, editPass);
}


BEGIN_MESSAGE_MAP(CProxySettings, CDialog)
    ON_BN_CLICKED(IDC_PROXY_ENABLE, &CProxySettings::OnBnClickedProxyEnable)
    ON_BN_CLICKED(IDC_PROXY_AUTH, &CProxySettings::OnBnClickedProxyAuth)
END_MESSAGE_MAP()


// CProxySettings message handlers

BOOL CProxySettings::OnInitDialog(){
    CString s1, s2;
    s1.LoadString(IDS_PROXY_SETTINGS2); SetWindowText(s1);
    CDialog::OnInitDialog();

    const char* proxyUser = NULL;
    const char* proxyPass = NULL;
    

    editUser.SetLimitText(EDIT_TEXT_MAXLENGTH);
    editPass.SetLimitText(EDIT_TEXT_MAXLENGTH);

    s1.LoadString(IDS_ACCOUNT_USEPROXY);  SetDlgItemText(IDC_PROXY_ENABLE, s1);
    s1.LoadString(IDS_ACCOUNT_PROXY_AUTH); SetDlgItemText(IDC_PROXY_AUTH, s1);
    s1.LoadString(IDS_ACCOUNT_PROXY_USER); SetDlgItemText(IDC_PROXY_STATIC_USER, s1);
    s1.LoadString(IDS_ACCOUNT_PROXY_PASS); SetDlgItemText(IDC_PROXY_STATIC_PASSWORD, s1);

    // read config from Registry 
    OutlookConfig* conf = (OutlookConfig*)getConfig();

    proxyUser = conf->getAccessConfig().getProxyUsername();
    proxyPass = conf->getAccessConfig().getProxyPassword();

    // proxy authentication
    if( (strcmp(proxyUser,"")) && (strcmp(proxyPass,"")) ){
        // proxy authentication required
        checkAuth.SetCheck(BST_CHECKED);
        editUser.EnableWindow(TRUE);
        editPass.EnableWindow(TRUE);
        GetDlgItem(IDC_PROXY_STATIC_USER)->EnableWindow(TRUE);
        GetDlgItem(IDC_PROXY_STATIC_PASSWORD)->EnableWindow(TRUE);

        // Note: use 'toWideChar' because we need UTF-8 conversion.
        WCHAR* tmp = toWideChar(proxyUser);
        s2 = tmp;
        SetDlgItemText(IDC_PROXY_EDIT_USER, s2);
        delete [] tmp;

        // Note: use 'toWideChar' because we need UTF-8 conversion.
        tmp = toWideChar(proxyUser);
        s2 = tmp; 
        SetDlgItemText(IDC_PROXY_EDIT_PASS, s2);
        delete [] tmp;
    }
    else{
        checkAuth.SetCheck(BST_UNCHECKED);
        editUser.EnableWindow(FALSE);
        editPass.EnableWindow(FALSE);
        GetDlgItem(IDC_PROXY_STATIC_USER)->EnableWindow(FALSE);
        GetDlgItem(IDC_PROXY_STATIC_PASSWORD)->EnableWindow(FALSE);
    }

    // use proxy
    if(conf->getAccessConfig().getUseProxy()){
        checkEnable.SetCheck(BST_CHECKED);
    }
    else{
        checkEnable.SetCheck(BST_UNCHECKED);
        checkAuth.EnableWindow(FALSE);
        editUser.EnableWindow(FALSE);
        editPass.EnableWindow(FALSE);
        GetDlgItem(IDC_PROXY_STATIC_USER)->EnableWindow(FALSE);
        GetDlgItem(IDC_PROXY_STATIC_PASSWORD)->EnableWindow(FALSE);
    }

    checkEnable.SetFocus();
    return FALSE;
}

void CProxySettings::OnBnClickedProxyEnable()
{
    if(checkEnable.GetCheck() == BST_CHECKED){
        checkAuth.EnableWindow(TRUE);
        if(checkAuth.GetCheck() == BST_CHECKED){
            editUser.EnableWindow(TRUE);
            editPass.EnableWindow(TRUE);
            GetDlgItem(IDC_PROXY_STATIC_USER)->EnableWindow(TRUE);
            GetDlgItem(IDC_PROXY_STATIC_PASSWORD)->EnableWindow(TRUE);   
        }
    }
    else{
        checkAuth.EnableWindow(FALSE);
        editUser.EnableWindow(FALSE);
        editPass.EnableWindow(FALSE);
        GetDlgItem(IDC_PROXY_STATIC_USER)->EnableWindow(FALSE);
        GetDlgItem(IDC_PROXY_STATIC_PASSWORD)->EnableWindow(FALSE);
    }
}

void CProxySettings::OnBnClickedProxyAuth()
{
    if(checkAuth.GetCheck() == BST_CHECKED){
        editUser.EnableWindow(TRUE);
        editPass.EnableWindow(TRUE);
        GetDlgItem(IDC_PROXY_STATIC_USER)->EnableWindow(TRUE);
        GetDlgItem(IDC_PROXY_STATIC_PASSWORD)->EnableWindow(TRUE);
    }
    else{
        editUser.EnableWindow(FALSE);
        editPass.EnableWindow(FALSE);
        GetDlgItem(IDC_PROXY_STATIC_USER)->EnableWindow(FALSE);
        GetDlgItem(IDC_PROXY_STATIC_PASSWORD)->EnableWindow(FALSE);
    }
}


void CProxySettings::OnOK(){
    CString proxyUser, proxyPassword;
    CString s1;
    _bstr_t bst;
    bool cancelSave = false;
    OutlookConfig* conf = (OutlookConfig*)getConfig();

    GetDlgItemText(IDC_PROXY_EDIT_USER,proxyUser);
    GetDlgItemText(IDC_PROXY_EDIT_PASS,proxyPassword);

    if( (checkEnable.GetCheck() == BST_CHECKED) && (checkAuth.GetCheck() == BST_CHECKED)){
        if (proxyUser == ""){
            s1.LoadString(IDS_ERROR_PROXY_USERNAME);
            MessageBox(s1);
            cancelSave = true;
            goto finally;
        };
        if (proxyPassword == ""){
            s1.LoadString(IDS_ERROR_PROXY_PASSWORD);
            MessageBox(s1);
            cancelSave = true;
            goto finally;
        };
    }

    if(checkEnable.GetCheck() == BST_CHECKED)
        conf->getAccessConfig().setUseProxy(TRUE);
    else
        conf->getAccessConfig().setUseProxy(FALSE);

    if(checkAuth.GetCheck() == BST_CHECKED){
        // Note: use 'toMultibyte' which uses charset UTF-8.
        char* tmp = toMultibyte(proxyUser.GetBuffer());
        if (tmp) {
            conf->getAccessConfig().setProxyUsername(tmp);
            delete [] tmp;
        }
        // Note: use 'toMultibyte' which uses charset UTF-8.
        tmp = toMultibyte(proxyPassword.GetBuffer());
        if (tmp) {
            conf->getAccessConfig().setProxyPassword(tmp);
            delete [] tmp;
        }
    }
    else{
        conf->getAccessConfig().setProxyUsername("");
        conf->getAccessConfig().setProxyPassword("");
    }

    // save values
  finally:
    if(!cancelSave){
        conf->save();

        CDialog::OnOK();
    }
    
}