 /*
 * Copyright (C) 2006-2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
*/

#include "vObjectConverter.h"
#include "outlook/utils.h"
#include "utils.h"

#include <string>
using namespace std;



// ------------------------------------------------------------------------------------------------------------
//                          ClientItem  ->  VObject  (client to server functions)
// ------------------------------------------------------------------------------------------------------------

/**
 * ClientContact (Outlook) -> VObject contact (wrapper of VCard 2.1).
 *
 * ClientContact properties are retrieved one by one, and inserted into VObject
 * structure. Quoted printable conversions are automatically done
 * inside VObject (when necessary).
 * Note: the returned VObject is new allocated, must be freed by the caller.
 *
 * @param cItem : the ClientContact object as input
 * @return      : the (new allocated) VObject contact
 */
VObject* client2VObjectContact(ClientContact* cItem) {

    VObject* vo = new VObject();

    wstring element;
    VProperty* vp    = NULL;
    bool value_found = false;
    bool email1   = false, email2   = false, email3 = false;
    bool homeTel1 = false, homeTel2 = false;
    bool busTel1  = false, busTel2  = false;


    // Properties that have NO correspondent in  VCard:
    // ------------------------------------------------
    // Anniversary
    // AssistantName
    // AssistantTelephoneNumber
    // Children
    // Department
    // OfficeLocation
    // Spouse
    // RadioTelephoneNumber
    // Yomi properties


    vp = new VProperty(L"BEGIN", L"VCARD");
    vo->addProperty(vp);
    delete vp; vp = NULL;

    vp = new VProperty(L"VERSION", L"2.1");
    vo->addProperty(vp);
    delete vp; vp = NULL;


    // ------- Name -------
    vp = new VProperty(L"N");

    element = cItem->getProperty(L"LastName");
    vp->addValue(element.c_str());

    element = cItem->getProperty(L"FirstName");
    vp->addValue(element.c_str());

    element = cItem->getProperty(L"MiddleName");
    vp->addValue(element.c_str());

    element = cItem->getProperty(L"Title");
    vp->addValue(element.c_str());

    element = cItem->getProperty(L"Suffix");
    vp->addValue(element.c_str());

    vo->addProperty(vp);
    delete vp; vp = NULL;


    // Formatted Name FN = <FileAs>
    element = cItem->getProperty(L"FileAs");
    vp = new VProperty(L"FN", element.c_str());
    vo->addProperty(vp);
    delete vp; vp = NULL;

    element = cItem->getProperty(L"Birthday");
    vp = new VProperty(L"BDAY", element.c_str());
    vo->addProperty(vp);
    delete vp; vp = NULL;

    element = cItem->getProperty(L"BusinessFaxNumber");
    vp = new VProperty(L"TEL", element.c_str());
    vp->addParameter(L"WORK", NULL);
    vp->addParameter(L"FAX", NULL);
    vo->addProperty(vp);
    delete vp; vp = NULL;

    element = cItem->getProperty(L"CompanyName");
    vp = new VProperty(L"ORG", element.c_str());
    vo->addProperty(vp);
    delete vp; vp = NULL;

    element = cItem->getProperty(L"Email1Address");
    if (element != EMPTY_WSTRING) {
        email1 = true;
        vp = new VProperty(L"EMAIL", element.c_str());
        vp->addParameter(L"INTERNET", NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    element = cItem->getProperty(L"Email2Address");
    if (element != EMPTY_WSTRING) {
        email2 = true;
        vp = new VProperty(L"EMAIL", element.c_str());
        vp->addParameter(L"INTERNET", NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    element = cItem->getProperty(L"Email3Address");
    if (element != EMPTY_WSTRING) {
        email3 = true;
        vp = new VProperty(L"EMAIL", element.c_str());
        vp->addParameter(L"INTERNET", NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    // Just send empty "EMAIL;INTERNET:" if no email found
    if (!email1 && !email2 && !email3) {
        vp = new VProperty(L"EMAIL");
        vp->addParameter(L"INTERNET", NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    element = cItem->getProperty(L"MobileTelephoneNumber");
    vp = new VProperty(L"TEL", element.c_str());
    vp->addParameter(L"CELL", NULL);
    vo->addProperty(vp);
    delete vp; vp = NULL;

    element = cItem->getProperty(L"PagerNumber");
    vp = new VProperty(L"TEL", element.c_str());
    vp->addParameter(L"PAGER", NULL);
    vo->addProperty(vp);
    delete vp; vp = NULL;

    element = cItem->getProperty(L"BusinessTelephoneNumber");
    if (element != EMPTY_WSTRING) {
        busTel1 = true;
        vp = new VProperty(L"TEL", element.c_str());
        vp->addParameter(L"WORK", NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    element = cItem->getProperty(L"Business2TelephoneNumber");
    if (element != EMPTY_WSTRING) {
        busTel2 = true;
        vp = new VProperty(L"TEL", element.c_str());
        vp->addParameter(L"WORK", NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    // Just send empty "TEL;WORK:" if no homeTel found
    if (!homeTel1 && !homeTel2) {
        vp = new VProperty(L"TEL");
        vp->addParameter(L"WORK", NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    element = cItem->getProperty(L"JobTitle");
    vp = new VProperty(L"TITLE", element.c_str());
    vo->addProperty(vp);
    delete vp; vp = NULL;

    element = cItem->getProperty(L"HomeTelephoneNumber");
    if (element != EMPTY_WSTRING) {
        homeTel1 = true;
        vp = new VProperty(L"TEL", element.c_str());
        vp->addParameter(L"HOME", NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    element = cItem->getProperty(L"Home2TelephoneNumber");
    if (element != EMPTY_WSTRING) {
        homeTel2 = true;
        vp = new VProperty(L"TEL", element.c_str());
        vp->addParameter(L"HOME", NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    // Just send empty "TEL;HOME:" if no homeTel found
    if (!homeTel1 && !homeTel2) {
        vp = new VProperty(L"TEL");
        vp->addParameter(L"HOME", NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    element = cItem->getProperty(L"CarTelephoneNumber");
    vp = new VProperty(L"TEL", element.c_str());
    vp->addParameter(L"CAR", NULL);
    vo->addProperty(vp);
    delete vp; vp = NULL;

    element = cItem->getProperty(L"HomeFaxNumber");
    vp = new VProperty(L"TEL", element.c_str());
    vp->addParameter(L"HOME", NULL);
    vp->addParameter(L"FAX", NULL);
    vo->addProperty(vp);
    delete vp; vp = NULL;

    element = cItem->getProperty(L"OtherTelephoneNumber");
    vp = new VProperty(L"TEL", element.c_str());
    vp->addParameter(L"VOICE", NULL);
    vo->addProperty(vp);
    delete vp; vp = NULL;

    element = cItem->getProperty(L"Categories");
    vp = new VProperty(L"CATEGORIES", element.c_str());
    vo->addProperty(vp);
    delete vp; vp = NULL;

    element = cItem->getProperty(L"WebPage");
    vp = new VProperty(L"URL", element.c_str());
    vo->addProperty(vp);
    delete vp; vp = NULL;


    // ----- Address HOME -----
    vp = new VProperty(L"ADR");
    vp->addParameter(L"HOME", NULL);
    value_found = false;

    element = cItem->getProperty(L"HomeAddressPostOfficeBox");
    value_found = true;
    vp->addValue(element.c_str());

    // empty value: "Extended Address" not present in SIF
    vp->addValue(EMPTY_WSTRING);

    element = cItem->getProperty(L"HomeAddressStreet");
    value_found = true;
    vp->addValue(element.c_str());

    element = cItem->getProperty(L"HomeAddressCity");
    value_found = true;
    vp->addValue(element.c_str());

    element = cItem->getProperty(L"HomeAddressState");
    value_found = true;
    vp->addValue(element.c_str());

    element = cItem->getProperty(L"HomeAddressPostalCode");
    value_found = true;
    vp->addValue(element.c_str());

    element = cItem->getProperty(L"HomeAddressCountry");
    value_found = true;
    vp->addValue(element.c_str());

    if(value_found) {
        vo->addProperty(vp);
    }
    delete vp; vp = NULL;


    // ----- Address -----
    vp = new VProperty(L"ADR");
    value_found = false;

    element = cItem->getProperty(L"OtherAddressPostOfficeBox");
    value_found = true;
    vp->addValue(element.c_str());

    // empty value: "Extended Address" not present in SIF
    vp->addValue(EMPTY_WSTRING);

    element = cItem->getProperty(L"OtherAddressStreet");
    value_found = true;
    vp->addValue(element.c_str());

    element = cItem->getProperty(L"OtherAddressCity");
    value_found = true;
    vp->addValue(element.c_str());

    element = cItem->getProperty(L"OtherAddressState");
    value_found = true;
    vp->addValue(element.c_str());

    element = cItem->getProperty(L"OtherAddressPostalCode");
    value_found = true;
    vp->addValue(element.c_str());

    element = cItem->getProperty(L"OtherAddressCountry");
    value_found = true;
    vp->addValue(element.c_str());

    if(value_found) {
        vo->addProperty(vp);
    }
    delete vp; vp = NULL;


    // ----- Address WORK -----
    vp = new VProperty(L"ADR");
    vp->addParameter(L"WORK", NULL);
    value_found = false;

    element = cItem->getProperty(L"BusinessAddressPostOfficeBox");
    value_found = true;
    vp->addValue(element.c_str());

    // empty value: "Extended Address" not present in SIF
    vp->addValue(EMPTY_WSTRING);

    element = cItem->getProperty(L"BusinessAddressStreet");
    value_found = true;
    vp->addValue(element.c_str());

    element = cItem->getProperty(L"BusinessAddressCity");
    value_found = true;
    vp->addValue(element.c_str());

    element = cItem->getProperty(L"BusinessAddressState");
    value_found = true;
    vp->addValue(element.c_str());

    element = cItem->getProperty(L"BusinessAddressPostalCode");
    value_found = true;
    vp->addValue(element.c_str());

    element = cItem->getProperty(L"BusinessAddressCountry");
    value_found = true;
    vp->addValue(element.c_str());

    if(value_found) {
        vo->addProperty(vp);
    }
    delete vp; vp = NULL;


    element = cItem->getProperty(L"Body");
    vp = new VProperty(L"NOTE", element.c_str());
    vo->addProperty(vp);
    delete vp; vp = NULL;


    vp = new VProperty(L"END", L"VCARD");
    vo->addProperty(vp);
    delete vp; vp = NULL;

    return vo;
}





/**
 * ClientAppointment (Outlook) -> VObject calendar (wrapper of VCal 1.0).
 *
 * ClientAppointment properties are retrieved one by one, and inserted into VObject
 * structure. Quoted printable conversions are automatically done
 * inside VObject (when necessary).
 * Note: the returned VObject is new allocated, must be freed by the caller.
 *
 * @param cItem : the ClientAppointment object as input
 * @return      : the (new allocated) VObject calendar
 */
VObject* client2VObjectAppointment(ClientAppointment* cItem) {

    VObject* vo = new VObject();

    VProperty* vp  = NULL;
    DATE startdate = NULL;
    DATE enddate   = NULL;
    wstring element;
    wstring dateValue;

    vp = new VProperty(TEXT("BEGIN"), TEXT("VCALENDAR"));
    vo->addProperty(vp);
    delete vp; vp = NULL;

    vp = new VProperty(TEXT("VERSION"), TEXT("1.0"));
    vo->addProperty(vp);
    delete vp; vp = NULL;

    vp = new VProperty(TEXT("BEGIN"), TEXT("VEVENT"));
    vo->addProperty(vp);
    delete vp; vp = NULL;

    element = cItem->getProperty(L"Subject");
    vp = new VProperty(TEXT("SUMMARY"), element.c_str());
    vo->addProperty(vp);
    delete vp; vp = NULL;

    element = cItem->getProperty(L"Location");
    vp = new VProperty(TEXT("LOCATION"), element.c_str());
    vo->addProperty(vp);
    delete vp; vp = NULL;

    element = cItem->getProperty(L"Body");
    vp = new VProperty(TEXT("DESCRIPTION"), element.c_str());
    vo->addProperty(vp);
    delete vp; vp = NULL;

    //
    // DTSTART, DTEND: server needs "yyyyMMdd" format if AllDayEvent
    //
    element = cItem->getProperty(L"Start");
    if (isSystemTimeDate(element)){
        systemTimeDateToDouble(element, &startdate);
        doubleToSystemTimeDateForVCal(element, startdate);       // if all-day-event: "yyyy-MM-dd" -> "yyyyMMdd"
    }
    else {
        systemTimeToDouble(element, &startdate, NULL);
    }
    vp = new VProperty(TEXT("DTSTART"), element.c_str());
    vo->addProperty(vp);
    delete vp; vp = NULL;

    element = cItem->getProperty(L"End");
    if (isSystemTimeDate(element)){
        systemTimeDateToDouble(element, &enddate);
        enddate += 1;                                            // Since 6.0.9: DTEND is +1day for all-day events!
        doubleToSystemTimeDateForVCal(element, enddate);         // if all-day-event: "yyyy-MM-dd" -> "yyyyMMdd"
    }
    else {
        systemTimeToDouble(element, &enddate, NULL);
    }
    vp = new VProperty(TEXT("DTEND"), element.c_str());
    vo->addProperty(vp);
    delete vp; vp = NULL;
    

    element = cItem->getProperty(L"Categories");
    vp = new VProperty(TEXT("CATEGORIES"), element.c_str());
    vo->addProperty(vp);
    delete vp; vp = NULL;


    long sensitivity;
    element = cItem->getProperty(L"Sensitivity");
    sensitivity = _wtoi(element.c_str());
    
    vp = new VProperty(TEXT("CLASS"));
    if(sensitivity == olPrivate)
        vp->addValue(TEXT("PRIVATE"));
    else
        vp->addValue(TEXT("PUBLIC"));

    vo->addProperty(vp);
    delete vp; vp = NULL;



    // Recipients
    ///////////////////////////////////////////////////////////////////////
    /********* To be developed *************

    IRecipients* pRecipients;
    pAppointment->get_Recipients(&pRecipients);
    int countRecipient = 0;
    pRecipients->get_Count(&countRecipient);

    for(int i = 0; i < countRecipient; i++) {
    IRecipient* pRecipient;
    pRecipients->Item(i + 1, &pRecipient);

    WCHAR* attedee = new WCHAR[100];
    pRecipient->get_Name(&element.c_str());
    element = cItem->getProperty(L"Subject");
    wcscpy(attedee, element.c_str());
    wcscpy(attedee, TEXT(" "));
    pRecipient->get_Address(&element.c_str());
    element = cItem->getProperty(L"Subject");
    wcscpy(attedee, element.c_str());

    vp = new VProperty(TEXT("ATTENDEE"), attedee);
    vo->addProperty(vp);

    delete vp;         vp      = NULL;
    delete [] attedee; attedee = NULL;
    pRecipient->Release();
    }

    pRecipients->Release();
    */
    ///////////////////////////////////////////////////////////////////////

    //
    // ReminderSet
    //
    BOOL bReminder;
    element = cItem->getProperty(L"ReminderSet");
    bReminder = _wtoi(element.c_str());

    if(bReminder == TRUE) {
        long minBefore;
        element = cItem->getProperty(L"ReminderMinutesBeforeStart");
        minBefore = _wtoi(element.c_str());
        double minStartDate = startdate * 1440;
        //subtract the alarm
        minStartDate -= minBefore;

        doubleToSystemTime(dateValue, minStartDate/1440, USE_UTC);

        vp = new VProperty(L"AALARM");
        vp->addValue(dateValue.c_str());            // "RunTime"
        vp->addValue(EMPTY_WSTRING);                // "Snooze Time" (empty)
        vp->addValue(L"0");                         // "Repeat Count"
        vp->addValue(L"reminder");                  // "Audio Content"
        vo->addProperty(vp);

        delete vp; vp = NULL;
    }
    else {
        // No reminder: send empty "AALARM:"
        vp = new VProperty(L"AALARM");
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }


    //
    // Recurrence pattern -> RRULE
    //
    BOOL isRec;
    element = cItem->getProperty(L"IsRecurring");
    isRec = _wtoi(element.c_str());
    
    if(isRec) {
        ClientRecurrence* cRec = cItem->getRecPattern();
        wstring rRule = clientRecurrenceToRRULE(cRec);

        if(rRule != EMPTY_WSTRING) {
            vp = new VProperty(TEXT("RRULE"), rRule.c_str());
            vo->addProperty(vp);
            delete vp; vp = NULL;
        }

        //
        // Exceptions: EXDATE
        //
        if (cRec) {
            BOOL isAllDay = cItem->isAllDayEvent();
            wstring start = cItem->getProperty(L"Start");
            int numExc    = cRec->getExceptionsCount();

            if (numExc > 0) {
                vp = new VProperty(TEXT("EXDATE"));

                for (int i=0; i<numExc; i++) {
                    ClientAppException* cEx = cRec->getException(i);
                    if (cEx) {
                        // Necessary to get the correct value! (bug on outlook for allday deleted)
                        wstring origDate = cEx->formatOriginalDate(isAllDay, start);
                        DATE exdate = NULL;

                        if (isAllDay) {
                            systemTimeDateToDouble(origDate, &exdate);
                            doubleToSystemTimeDateForVCal(element, exdate);       // if all-day-event: "yyyy-MM-dd" -> "yyyyMMdd"
                        }
                        else {
                            element = origDate;                                   // UTC value: "YYYYMMDDThhmmssZ"
                        }
                        vp->addValue(element.c_str());
                    }
                }
                vo->addProperty(vp);
                delete vp; vp = NULL;
            }
        }
    }
    else {
        // Not recurring: send empty "RRULE:"
        vp = new VProperty(TEXT("RRULE"));
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }


    vp = new VProperty(TEXT("END"), TEXT("VEVENT"));
    vo->addProperty(vp);
    delete vp; vp = NULL;

    vp = new VProperty(TEXT("END"), TEXT("VCALENDAR"));
    vo->addProperty(vp);
    delete vp; vp = NULL;

    return vo;
}






// ------------------------------------------------------------------------------------------------------------
//                            VObject  ->  ClientItem  (server to client functions)
// ------------------------------------------------------------------------------------------------------------

/**
 * VObject contact (wrapper of VCard) -> ClientContact (Outlook).
 * Fills the given ClientContact 'cItem' with properties from VObject 'vo'.
 *
 * @param vo    : [INPUT] the VObject contact object
 * @param cItem : the ClientContact object to fill
 */
void vObject2ClientContact(VObject *vo, ClientContact* cItem) {

	int businessTel  = 0;
	int homeTel		 = 0;
	int email		 = 0;
	WCHAR* name      = NULL;
    WCHAR* element   = NULL;

    // 
	// Cycle on each VProperty:
	//
    for(int i=0; i < vo->propertiesCount(); i++) {

		name    = vo->getProperty(i)->getName();
        element = vo->getProperty(i)->getValue(0);      // This is the first value of the property.

		if(!wcscmp(name, L"N")) {
			if(element = vo->getProperty(i)->getPropComponent(1))
                cItem->setProperty(L"LastName", element);
			if(element = vo->getProperty(i)->getPropComponent(2))
				cItem->setProperty(L"FirstName", element);
			if(element = vo->getProperty(i)->getPropComponent(3))
				cItem->setProperty(L"MiddleName", element);
			if(element = vo->getProperty(i)->getPropComponent(4))
				cItem->setProperty(L"Title", element);
			if(element = vo->getProperty(i)->getPropComponent(5))
				cItem->setProperty(L"Suffix", element);
		}

		else if(!wcscmp(name, L"FN"))
			cItem->setProperty(L"FileAs", element);

		else if(!wcscmp(name, L"BDAY"))
			cItem->setProperty(L"Birthday", element);

		else if(!wcscmp(name, L"TEL") && vo->getProperty(i)->containsParameter(L"WORK")) {
            if (vo->getProperty(i)->containsParameter(L"FAX")) {
				cItem->setProperty(L"BusinessFaxNumber", element);
            }
            else if (vo->getProperty(i)->containsParameter(L"CELL")) { 
            }
            else {
				if(businessTel == 0) {
					cItem->setProperty(L"BusinessTelephoneNumber", element);
					businessTel++;
				}
				else{
					cItem->setProperty(L"Business2TelephoneNumber", element);
				}
            }
		}

		else if(!wcscmp(name, L"ORG")) {
			cItem->setProperty(L"CompanyName", element);
		}
		else if(!wcscmp(name, L"EMAIL")) {
			if(email == 0) {
				cItem->setProperty(L"Email1Address", element);
				email++;
			}
			else if(email == 1) {
				email++;
				cItem->setProperty(L"Email2Address", element);
			}
			else
				cItem->setProperty(L"Email3Address", element);
		}

		else if(!wcscmp(name, L"TEL") && vo->getProperty(i)->containsParameter(L"CELL"))
			cItem->setProperty(L"MobileTelephoneNumber", element);

		else if(!wcscmp(name, L"TEL") && vo->getProperty(i)->containsParameter(L"PAGER"))
			cItem->setProperty(L"PagerNumber", element);

		else if(!wcscmp(name, L"TITLE"))
			cItem->setProperty(L"JobTitle", element);

		else if(!wcscmp(name, L"TEL") && vo->getProperty(i)->containsParameter(L"HOME")) {
            if(vo->getProperty(i)->containsParameter(L"FAX")) {
				cItem->setProperty(L"HomeFaxNumber", element);
            }
            else if(vo->getProperty(i)->containsParameter(L"CELL")) {
            }
			else if(homeTel == 0) {
				cItem->setProperty(L"HomeTelephoneNumber", element);
				homeTel++;
			}
            else {
				cItem->setProperty(L"Home2TelephoneNumber", element);
            }
		}

		else if(!wcscmp(name, L"TEL") && vo->getProperty(i)->containsParameter(L"CAR"))
			cItem->setProperty(L"CarTelephoneNumber", element);

		else if(!wcscmp(name, L"TEL") && vo->getProperty(i)->containsParameter(L"VOICE"))
			cItem->setProperty(L"OtherTelephoneNumber", element);

		else if(!wcscmp(name, L"CATEGORIES"))
			cItem->setProperty(L"Categories", element);

		else if(!wcscmp(name, L"URL"))
			cItem->setProperty(L"WebPage", element);

		else if(!wcscmp(name, L"NOTE"))
			cItem->setProperty(L"Body", element);

        else if(!wcscmp(name, L"ADR")) {
			if(vo->getProperty(i)->containsParameter(L"HOME")) {
				if(element = vo->getProperty(i)->getPropComponent(1))
					cItem->setProperty(L"HomeAddressPostOfficeBox", element);
				if(element = vo->getProperty(i)->getPropComponent(3))
					cItem->setProperty(L"HomeAddressStreet", element);
				if(element = vo->getProperty(i)->getPropComponent(4))
					cItem->setProperty(L"HomeAddressCity", element);
				if(element = vo->getProperty(i)->getPropComponent(5))
					cItem->setProperty(L"HomeAddressState", element);
				if(element = vo->getProperty(i)->getPropComponent(6))
					cItem->setProperty(L"HomeAddressPostalCode", element);
				if(element = vo->getProperty(i)->getPropComponent(7))
					cItem->setProperty(L"HomeAddressCountry", element);
			}
			else if(vo->getProperty(i)->containsParameter(L"WORK")) {
				if(element = vo->getProperty(i)->getPropComponent(1))
					cItem->setProperty(L"BusinessAddressPostOfficeBox", element);
				if(element = vo->getProperty(i)->getPropComponent(3))
					cItem->setProperty(L"BusinessAddressStreet", element);
				if(element = vo->getProperty(i)->getPropComponent(4))
					cItem->setProperty(L"BusinessAddressCity", element);
				if(element = vo->getProperty(i)->getPropComponent(5))
					cItem->setProperty(L"BusinessAddressState", element);
				if(element = vo->getProperty(i)->getPropComponent(6))
					cItem->setProperty(L"BusinessAddressPostalCode", element);
				if(element = vo->getProperty(i)->getPropComponent(7))
					cItem->setProperty(L"BusinessAddressCountry", element);
			}
			else {
                if(element = vo->getProperty(i)->getPropComponent(1))
					cItem->setProperty(L"OtherAddressPostOfficeBox", element);
				if(element = vo->getProperty(i)->getPropComponent(3))
					cItem->setProperty(L"OtherAddressStreet", element);
				if(element = vo->getProperty(i)->getPropComponent(4))
					cItem->setProperty(L"OtherAddressCity", element);
				if(element = vo->getProperty(i)->getPropComponent(5))
					cItem->setProperty(L"OtherAddressState", element);
				if(element = vo->getProperty(i)->getPropComponent(6))
					cItem->setProperty(L"OtherAddressPostalCode", element);
				if(element = vo->getProperty(i)->getPropComponent(7))
					cItem->setProperty(L"OtherAddressCountry", element);
			}
        }
    }
}





/**
* VObject contact (wrapper of VCal) -> ClientContact (Outlook).
* Fills the given ClientAppointment 'cItem' with properties from VObject 'vo'.
*
* @param vo    : [INPUT] the VObject contact object
* @param cItem : the ClientAppointment object to fill
*/
void vObject2ClientAppointment(VObject *vo, ClientAppointment* cItem) {

    WCHAR* element = NULL;
    WCHAR tempbuf[128];

    DATE startdate = NULL;
    DATE enddate   = NULL;
    wstring startDateValue;
    wstring endDateValue;


    if(element = getVObjectPropertyValue(vo, L"SUMMARY")) {
        cItem->setProperty(L"Subject", element);
    }
    if(element = getVObjectPropertyValue(vo, L"LOCATION")) {
        cItem->setProperty(L"Location", element);
    }
    if(element = getVObjectPropertyValue(vo, L"DESCRIPTION")) {
        cItem->setProperty(L"Body", element);
    }

    //
    // DTSTART, DTEND:
    // Set the start and end date. If the start is 00:00 and end is 23:59 the appointment is decided to be
    // an all day event. So the AllDayEvent tag is put to 'true' only in the SIF structure.
    //
    if(element = getVObjectPropertyValue(vo, L"DTSTART")){
        systemTimeToDouble(element, &startdate, NULL);                  // 'startdate' converted from UTC to local (if necessary)
        startDateValue = element;
    }

    if(element = getVObjectPropertyValue(vo, L"DTEND")) {
        systemTimeToDouble(element, &enddate, NULL);                    // 'enddate' converted from UTC to local (if necessary)
        endDateValue = element;
    }

    // Check if it's an all-day-event interval (00:00 - 23:59).
    BOOL isAllDay = isAllDayEventInterval(startdate, enddate);
    // Alternate form: (check format "yyyyMMdd")
    if (!isAllDay) {
        if (startDateValue.size() == 8 && endDateValue.size() == 8 ) {
            if ((enddate - 1) >= startdate) {                           // Since 6.0.9: DTEND is +1day for all-day events!
                enddate -= 1;                                           // (also make sure we don't get a endDate before startDate with this fix)
            }
            isAllDay = true;
        }
    }
    
    if (startdate){
        if (isAllDay){
            doubleToSystemTimeDate(startDateValue, startdate);          // UTC ignored (only date)
            cItem->setProperty(L"Start", startDateValue);               // "yyyy-MM-dd" (all-day)
        }
        else {
            element = vo->getProperty(TEXT("DTSTART"))->getValue();
            cItem->setProperty(L"Start", element);                      // "YYYYMMDDThhmmssZ"
        }
    }
    if (enddate){
        if (isAllDay) {
            doubleToSystemTimeDate(endDateValue, enddate);              // UTC ignored (only date)
            cItem->setProperty(L"End", endDateValue);                   // "yyyy-MM-dd" (all-day)
        }
        else {
            element = vo->getProperty(TEXT("DTEND"))->getValue();
            cItem->setProperty(L"End", element);                        // "YYYYMMDDThhmmssZ"
        }
    }
    // Note: "AllDayEvent" MUST be set AFTER the "Start" and "End" fields to be correctly managed
    cItem->setProperty(L"AllDayEvent", isAllDay? L"1" : L"0");


    if(element = getVObjectPropertyValue(vo, L"CATEGORIES")) {
        cItem->setProperty(L"Categories", element);
    }

    if(element = getVObjectPropertyValue(vo, L"CLASS")) {
        if(!wcscmp(element, TEXT("PRIVATE")) || !wcscmp(element, TEXT("CONFIDENTIAL")))
            wsprintf(tempbuf, TEXT("%i"), olPrivate);		// Private = 2
        else
            wsprintf(tempbuf, TEXT("%i"), olNormal);		// Normal = 0
    }
    else {
        wsprintf(tempbuf, TEXT("%i"), olNormal);			// Normal = 0
    }
    cItem->setProperty(L"Sensitivity", tempbuf);


    /****************** Decomment here when recipients supported ************************
    //ATTENDEE;ROLE=OWNER;STATUS=CONFIRMED:John Smith <jsmith@host1.com>
    //ATTENDEE;ROLE=ATTENDEE;STATUS=CONFIRMED;VALUE=URL;TYPE=VCARD:http://www.xyz.com/~myvcard.vcf

    for(i = 0; i<vo->propertiesCount(); i++) {
    if(!wcscmp(vo->getProperty(i)->getName(), TEXT("ATTENDEE")) &&
    vo->getProperty(i)->getValue() &&
    !vo->getProperty(i)->containsParameter(TEXT("VALUE"))) {

    element = vo->getProperty(i)->getValue();

    WCHAR* attendee = new WCHAR[200];
    wcscpy(attendee, vo->getProperty(i)->getValue());
    WCHAR separator[] = TEXT("<");

    WCHAR* pos = NULL;
    pos = wcsstr(attendee, separator);
    if(pos) {
    WCHAR* name = new WCHAR[100];
    wcscpy(name, TEXT(""));
    if((pos - attendee)-1 > 0) {
    wcsncpy(name, attendee, (pos - attendee)-1);
    name[pos - attendee] = TEXT('\0');
    }
    WCHAR* addr = new WCHAR[100];
    wcscpy(addr, pos+1);
    addr[wcslen(addr) - 1] = TEXT('\0');

    cItem->setProperty(L"Address", addr);
    cItem->setProperty(L"Name", name);

    delete [] name;  name = NULL;
    delete [] addr;  addr = NULL;
    }
    }
    }
    cItem->setProperty(L"Recipient", element);
    ***********************************************************************/

    //
    // AALARM
    // The value consists of: Run Time, Snooze Time, Repeat Count, Audio Content
    //
    if(element = getVObjectPropertyValue(vo, L"AALARM")) {

        cItem->setProperty(L"ReminderSet", L"1");
        DATE runTime;
        systemTimeToDouble(vo->getProperty(TEXT("AALARM"))->getPropComponent(1), &runTime);

        long minBeforeEvent = round((startdate - runTime) * 1440);
        // Safety check: values < 0 not accepted.
        if (minBeforeEvent < 0) {
            minBeforeEvent = 0;
        }
        wsprintf(tempbuf, TEXT("%i"), minBeforeEvent);
        cItem->setProperty(L"ReminderMinutesBeforeStart", tempbuf);
    }
    else {
        // AALARM not found -> no reminder.
        cItem->setProperty(L"ReminderSet", L"0");
    }


    //
    // RRULE -> Recurrence pattern
    //
    if(element = getVObjectPropertyValue(vo, L"RRULE")) {
        wstring rrule = element;

        cItem->setProperty(L"IsRecurring", L"1");               // This also enables the recPattern instance...
        ClientRecurrence* cRec = cItem->getRecPattern();        // 'cRec' is a pointer to the internal recPattern
        rRULEToClientRecurrence(rrule, cRec, startdate);        // Fill all cRec members.

        //
        // Recurrence exceptions:
        // ----------------------
        list<wstring> excludeDates, includeDates;
        excludeDates.clear();
        includeDates.clear();

        // EXDATE -> occurrences to delete
        VProperty* vprop = vo->getProperty(L"EXDATE");
        if(vprop) {
            int i=0;
            while(element = vprop->getValue(i)) {
                if (wcslen(element) > 0) {
                    excludeDates.push_back(element);
                }
                i++;
            }
        }

        // RDATE -> occurrences to add
        vprop = vo->getProperty(L"RDATE");
        if(vprop) {
            int i=0;
            while(element = vprop->getValue(i)) {
                if (wcslen(element) > 0) {
                    includeDates.push_back(element);
                }
                i++;
            }
        }

        // Add exceptions to Outlook: new events could be created in Outlook
        // (added occurrences are not supperted by Outlook)
        setRecurrenceExceptions(cItem, excludeDates, includeDates);
    }

    // Not recurring.
    else {
        cItem->setProperty(L"IsRecurring", L"0");
    }
}



/**
 * Utility to safe-retrieve the property value inside VObject 'vo'.
 * @param vo           : VObject to read from
 * @param propertyName : the property name requested
 * @return             : the property value (NULL if not found)
 */
WCHAR* getVObjectPropertyValue(VObject* vo, const WCHAR* propertyName) {

    WCHAR* propertyValue = NULL;
    VProperty* vprop = vo->getProperty(propertyName);
    if (vprop && vprop->getValue()) {
        propertyValue = vprop->getValue();
    }
    return propertyValue;
}


/**
 * Check the productID and version of VObject passed.
 * - 'productID' MUST be correct for itemType passed (e.g. "VCARD" for contacts) 
 * - 'version' is only checked to be the one supported (log info if wrong)
 *
 * @param  vo         the VObject to check
 * @param  itemType   the itemType in use (contacts, tasks, ...)
 * @return            true if productID is correct
 */
bool checkVobjectTypeAndVersion(VObject* vo, const wstring& itemType) {

    WCHAR* prodID  = vo->getProdID();
    WCHAR* version = vo->getVersion();
    if (!prodID || !version) {
        return false;
    }

    if (itemType == CONTACT) {
        if (wcscmp(prodID, L"VCARD")) {
            LOG.error(ERR_ITEM_VOBJ_WRONG_TYPE, prodID, L"VCARD");
            return false;
        }
        if (wcscmp(version, TEXT(VCARD_VERSION_SUPPORTED))) {
            // Just log a warning...
            LOG.info(INFO_ITEM_VOBJ_WRONG_VERSION, version, TEXT(VCARD_VERSION_SUPPORTED));
            //return false;
        }
    }

    else if (itemType == APPOINTMENT) {
        if (wcscmp(prodID, L"VCALENDAR")) {
            LOG.error(ERR_ITEM_VOBJ_WRONG_TYPE, prodID, L"VCALENDAR");
            return false;
        }
        if (wcscmp(version, TEXT(ICAL_VERSION_SUPPORTED))) {
            // Just log a warning...
            LOG.info(INFO_ITEM_VOBJ_WRONG_VERSION, version, TEXT(ICAL_VERSION_SUPPORTED));
            //return false;
        }
    }

    // *** TODO ***
    //else if (itemType == TASK) {
    //}

    return true;
}

