 /*
 * Copyright (C) 2006-2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
*/

#ifndef INCL_CLIENTAPPEXCEPTION
#define INCL_CLIENTAPPEXCEPTION


#include "outlook/defs.h"

#include <string>



///////////////////////////////////////////////////////////////////
class ClientAppException {

private:

    // Pointer to microsoft outlook objects.
    ExceptionPtr          pException;                   // the exception
    _AppointmentItemPtr   pAppointment;                 // the occurrence of the exception (an appointment item)


    // Occurrence properties:
    std::wstring  subject;
    std::wstring  body;
    std::wstring  location;
    std::wstring  start;
    std::wstring  end;
    std::wstring  allDayEvent;
    std::wstring  busyStatus;
    std::wstring  reminderSet;
    std::wstring  reminderMinutesBeforeStart;
    std::wstring  importance;

    // Exception properties:
    DATE          originalDate;
    BOOL          deleted;


    // Internal use: true if values are updated with Outlook.
    // Used because all values need to be retrieved together, so this flag
    // notifies when it's necessary to update properties.
    bool isUpdated;

    // Internal use: true if this exception has been saved to Outlook.
    bool saved;


public:

    // Constructor
    ClientAppException();

    // Destructor
    ~ClientAppException();


    // Set COM pointers to this object.
    // One of these two method MUST be called before using the class
    void setCOMPtr(ExceptionPtr& ptr);
    void setCOMPtr(_AppointmentItemPtr& ptr);


    // Read ALL properties from Outlook.
    int read();
    // Save the exception to Outlook: save the modified occurrence.
    int saveOccurrence();
    // Save the exception to Outlook: delete the occurrence.
    int deleteOccurrence();


    // Returns true if this exception has already been saved.
    bool isSaved();


    //
    // Methods to manage the exception properties. 
    // -------------------------------------------
    // To get appointment exceptions (READ) call 'read()' and use 'getException(index)' 
    // methods of ClientRecurrence and browse through the list of exceptions.
    // To set an appointment exception (WRITE) set all properties and then call
    // 'saveAllExceptions()' method of ClientRecurrence.
    //

    // get/set exception properties
    const BOOL getDeleted     ();
    const DATE getOriginalDate();
    void       setDeleted     (const BOOL);
    void       setOriginalDate(DATE val);


    // To quickly get/set occurrence properties defined in 'exAppointmentFields[]'
    const std::wstring  getAppProperty(const std::wstring& propertyName);
    int                 setAppProperty(const std::wstring& propertyName, const std::wstring& propertyValue);


    // get/set single occurrence properties
    const std::wstring   getSubject                   ();
    const std::wstring   getBody                      ();
    const std::wstring   getLocation                  ();
    const std::wstring   getStart                     ();
    const std::wstring   getEnd                       ();
    const std::wstring   getAllDayEvent               ();
    const std::wstring   getBusyStatus                ();
    const std::wstring   getReminderSet               ();
    const std::wstring   getReminderMinutesBeforeStart();
    const std::wstring   getImportance                ();

    void setSubject                   (const std::wstring& val);
    void setBody                      (const std::wstring& val);
    void setLocation                  (const std::wstring& val);
    void setStart                     (const std::wstring& val);
    void setEnd                       (const std::wstring& val);
    void setAllDayEvent               (const std::wstring& val);
    void setBusyStatus                (const std::wstring& val);
    void setReminderSet               (const std::wstring& val);
    void setReminderMinutesBeforeStart(const std::wstring& val);
    void setImportance                (const std::wstring& val);


    // Returns "OriginalDate" property in string format.
    const std::wstring  formatOriginalDate(BOOL isAllDay, const std::wstring& start);
    const std::wstring  formatOriginalDate();

};

#endif
