 /*
 * Copyright (C) 2006-2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
*/

#ifndef INCL_CLIENTCONTACT
#define INCL_CLIENTCONTACT


#include "outlook/defs.h"
#include "outlook/ClientItem.h"

#include <string>



///////////////////////////////////////////////////////////////////
class ClientContact : public ClientItem {

private:
    
    // Pointer to microsoft outlook objects.
    _ContactItemPtr  pContact;

    // Pointer to Redemption safe objects.
    Redemption::ISafeContactItemPtr pSafeContact;

    // Used to know if Outlook will automatically create events during saveItem().
    bool willCreateAnniversaryEvent;
    bool willCreateBirthdayEvent;

    
    void createSafeContactInstance();

    bool isSecureProperty (const std::wstring& propertyName);
    bool isComplexProperty(const std::wstring& propertyName);

    const std::wstring getSafeProperty   (const std::wstring& propertyName);
    const std::wstring getComplexProperty(const std::wstring& propertyName);

    int setComplexProperty(const std::wstring& propertyName, const std::wstring& propertyValue);

    // Internal utility: get the correct format of firstName and lastName fields.
    const std::wstring getCorrectNameValue(const std::wstring& name, const std::wstring& fullName);

public:

    // Constructor
    ClientContact();
    ClientContact(const ClientContact& c);
    ClientContact operator=(const ClientContact& c);

    // Destructor
    ~ClientContact();

    // Set a COM pointer to this object: refresh all members.
    void setCOMPtr(_ContactItemPtr& ptr, const std::wstring& itemID);
    void setCOMPtr(_ContactItemPtr& ptr);

    // Returns a reference to the internal COM pointer.
    _ContactItemPtr& getCOMPtr();


    //
    // Client operations on the item.
    //
    int saveItem();
    int deleteItem();
    ClientItem* copyItem();
    //int moveItem(ClientFolder* destFolder);


    // Return true if correspondent event has been created by Outlook during saveItem().
    bool createdAnniversaryEvent();
    bool createdBirthdayEvent();
};

#endif
