 /*
 * Copyright (C) 2006-2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
*/

#ifndef INCL_CLIENTITEM
#define INCL_CLIENTITEM


#include "outlook/defs.h"

#include <map>
#include <string>



/*
******************************************************************************
* Class ClientItem: 
* 
*
* Class methods automatically catch and manage COM pointers exceptions
* Class methods throw ClientException pointer in case of error
******************************************************************************
*/
class ClientItem {

protected:

    std::wstring ID;            // The item entry-ID.
    std::wstring itemType;      // The item-type of the item (contact/task/mail...)
    std::wstring parentPath;    // The full path of parent folder.


    // Map of item properties: <name, index>
    std::map<std::wstring,int> propertyMap;
    int propertiesCount;
    int propertiesIndex;


    // Pointers to microsoft outlook objects.
    ItemPropertiesPtr   pItemProperties;
    ItemPropertyPtr     pItemProperty;


    // Result of COM pointers operations.
    HRESULT hr;


    void createPropertyMap();
    const std::wstring convertPropertyName(const std::wstring& SIFName);

    virtual bool isSecureProperty  (const std::wstring& propertyName) = 0;
    virtual bool isComplexProperty (const std::wstring& propertyName) = 0;

    const         std::wstring getSimpleProperty (const std::wstring& propertyName);
    virtual const std::wstring getSafeProperty   (const std::wstring& propertyName) = 0;
    virtual const std::wstring getComplexProperty(const std::wstring& propertyName) = 0;

    int         setSimpleProperty (const std::wstring& propertyName, const std::wstring& propertyValue);
    virtual int setComplexProperty(const std::wstring& propertyName, const std::wstring& propertyValue) = 0;

public:

    // Constructor
    ClientItem();

    // Destructor
    ~ClientItem();


    const std::wstring& getID();
    const std::wstring& getType();
    const std::wstring& getParentPath();


    //
    // Client operations on the item.
    //
    virtual int         saveItem() = 0;
    virtual int       deleteItem() = 0;
    virtual ClientItem* copyItem() = 0;
    //virtual int       moveItem(ClientFolder* destFolder) = 0;
    // saveAs/close...


    //
    // Methods to menage item properties.
    //
    const int getPropertiesCount();
    const int getPropertiesIndex();

    // To retrieve a property value from its name.
    const std::wstring getProperty(const std::wstring& propertyName);
    // To set a property value from its name.
    int setProperty(const std::wstring& propertyName, const std::wstring& propertyValue);

};

#endif
