 /*
 * Copyright (C) 2006-2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
*/

#ifndef INCL_CLIENTTASK
#define INCL_CLIENTTASK


#include "outlook/defs.h"
#include "outlook/ClientItem.h"
#include "outlook/ClientRecurrence.h"

#include <string>



///////////////////////////////////////////////////////////////////
class ClientTask : public ClientItem {

private:

    // Pointer to microsoft outlook objects.
    _TaskItemPtr  pTask;

    // Pointer to Redemption safe objects.
    Redemption::ISafeTaskItemPtr pSafeTask;

    // Internal Object: to manage recurrence pattern.
    ClientRecurrence recPattern;


    void createSafeTaskInstance();
    void initializeRecPattern();


    bool isSecureProperty (const std::wstring& propertyName);
    bool isComplexProperty(const std::wstring& propertyName);

    const std::wstring getSafeProperty      (const std::wstring& propertyName);
    const std::wstring getComplexProperty   (const std::wstring& propertyName);
    const std::wstring getRecurrenceProperty(const std::wstring& propertyName);

    int setComplexProperty   (const std::wstring& propertyName, const std::wstring& propertyValue);
    int setRecurrenceProperty(const std::wstring& propertyName, const std::wstring& propertyValue);

public:

    // Constructor
    ClientTask();
    ClientTask(const ClientTask& c);
    ClientTask operator=(const ClientTask& c);

    // Destructor
    ~ClientTask();


    // Set a COM pointer to this object: refresh all members.
    void setCOMPtr(_TaskItemPtr& ptr, const std::wstring& itemID);
    void setCOMPtr(_TaskItemPtr& ptr);

    // Returns a reference to the internal COM pointer.
    _TaskItemPtr& getCOMPtr();
    
    // Returns a pointer to the internal ClientRecurrence object (NULL if not recurring).
    ClientRecurrence* getRecPattern();

    //
    // Client operations on the item.
    //
    int saveItem();
    int deleteItem();
    ClientItem* copyItem();
    //int moveItem(ClientFolder* destFolder);

};

#endif
